--
-- HelperAdminGui
-- V1.0.0.0
--
-- @author apuehri
-- Thanks to sfcmod for the inspiration, and code elements 
-- @date 16/12/2021
--
-- Copyright (C) apuehri
-- V1.0.0.0 ..... LS22 first implementation

haGui = {};
local haGui_mt = Class(haGui, TabbedMenu);

haGui.CONTROLS = {
	'pagehaGeneralSettings',
	'pagehaHelperName',
}

function haGui:new(messageCenter, l18n, inputManager)
    local self = TabbedMenu.new(nil, haGui_mt, messageCenter, l18n, inputManager);

	self:registerControls(haGui.CONTROLS);

	self.messageCenter = messageCenter;
	self.l18n = l18n;
	self.inputManager = g_inputBinding;

    return self;	
end;

function haGui:onGuiSetupFinished()
    haGui:superClass().onGuiSetupFinished(self)

	self.clickBackCallback = self:makeSelfCallback(self.onButtonBack);

	self.pagehaGeneralSettings:initialize();
	self.pagehaHelperName:initialize();

	self:initData();	

    self:setupPages();

	self:setupMenuButtonInfo();
end;

function haGui:setupPages()
    local pages = {
        {self.pagehaGeneralSettings, 'tab_helperAdmin.png'},
		{self.pagehaHelperName, 'tab_helperAdminNames.png'},
	}

    for i, _page in ipairs(pages) do
        local page, icon = unpack(_page);
        self:registerPage(page, i);
        self:addPageTab(page, helperadmin.dir .. 'icons/' .. icon);
    end
end;

function haGui:initData()
	haGui.actGuiNumHelper = (g_currentMission.maxNumHirables / 2);
	
	haGui.actGuiNameMHelp = {};
	haGui.actGuiNameWHelp = {};
	for i = 1, (helperadmin.maxHelper / 2), 1 do
		if (helperadmin.nameMan[i] == nil) or (helperadmin.nameMan[i] == "") then
			haGui.actGuiNameMHelp[i] = "Man"..tostring(i);
		else
			haGui.actGuiNameMHelp[i] = helperadmin.nameMan[i];
		end;
		if (helperadmin.nameWoman[i] == nil) or (helperadmin.nameWoman[i] == "") then
			haGui.actGuiNameWHelp[i] = "Woman"..tostring(i);
		else
			haGui.actGuiNameWHelp[i] = helperadmin.nameWoman[i];
		end;		
	end;
		
	haGui.actPricePerHour = helperadmin.pricePerHour;
	haGui.actOvertimeFact1 = helperadmin.overtimeFact1;
	haGui.actOvertimeFact2 = helperadmin.overtimeFact2;
	
	haGui.actGuiRideHorses = helperadmin.rideHorses;	
	
	haGui.actGuiShowHelp = helperadmin.showHelp;	

	if helperadmin.debug then
		print("--- HelperAdminGui Debug ... haGui:initData");		
	end;	
	
	if helperadmin.debug then
		print("--- HelperAdmin Debug ... haGui:initData(haGui.actGuiNumHelper = "..tostring(haGui.actGuiNumHelper)..")");
		print("--- HelperAdmin Debug ... haGui:initData(helperadmin.nameMan)");
		DebugUtil.printTableRecursively(helperadmin.nameMan,"*n* ", 0, 1);
		print("--- HelperAdmin Debug ... haGui:initData(helperadmin.nameWoman)");
		DebugUtil.printTableRecursively(helperadmin.nameWoman,"*n* ", 0, 1);
		print("--- HelperAdmin Debug ... haGui:initData(haGui.actPricePerHour = "..tostring(haGui.actPricePerHour)..")");
		print("--- HelperAdmin Debug ... haGui:initData(haGui.actOvertimeFact1 = "..tostring(haGui.actOvertimeFact1)..")");
		print("--- HelperAdmin Debug ... haGui:initData(haGui.actOvertimeFact2 = "..tostring(haGui.actOvertimeFact2)..")");
		print("--- HelperAdmin Debug ... haGui:initData(haGui.actGuiRideHorses = "..tostring(haGui.actGuiRideHorses)..")");
		print("--- HelperAdmin Debug ... haGui:initData(haGui.actGuiShowHelp = "..tostring(haGui.actGuiShowHelp)..")");
	end;	
end;

function haGui:setupMenuButtonInfo()
    local onButtonBackFunction = self.clickBackCallback;
	
    self.defaultMenuButtonInfo = {
		{inputAction = InputAction.MENU_BACK, text = g_i18n:getText("gui_hat_MenuExit"), callback = onButtonBackFunction},		
	}
    self.defaultMenuButtonInfoByActions[InputAction.MENU_BACK] = self.defaultMenuButtonInfo[1];
	
    self.defaultButtonActionCallbacks = {
		[InputAction.MENU_BACK] = onButtonBackFunction,
	}
end

function haGui:exitMenu()
	self:initData();
    self:changeScreen();
end
