--
-- HelperAdminGui
-- GeneralSettingsFrame
-- V1.0.0.0
--
-- @author apuehri
-- Thanks to sfcmod for the inspiration, and code elements 
-- @date 16/12/2021
--
-- Copyright (C) apuehri
-- V1.0.0.0 ..... LS22 first implementation

haGuiGeneralSettingsFrame = {};
local haGuiGeneralSettingsFrame_mt = Class(haGuiGeneralSettingsFrame, TabbedMenuFrameElement);

haGuiGeneralSettingsFrame.CONTROLS = {
    'guiNumHelper',
	'guiPricePerHour',
	'guiOvertimeFact1',
	'guiOvertimeFact2',
	'guiRideHorses',
	'guiShowHelp',
	'guiClickSave',
	
	'frameHeader',
    'boxLayout'
}

haGuiGeneralSettingsFrame.pricePerHour = {150;200;250;300;500;1000;2000;};
haGuiGeneralSettingsFrame.overTimeFact1 = {1.0;1.2;1.5;1.8;};
haGuiGeneralSettingsFrame.overTimeFact2 = {1.0;2.2;2.8;3.4;};

function haGuiGeneralSettingsFrame:new(i18n)
    local self = TabbedMenuFrameElement.new(nil, haGuiGeneralSettingsFrame_mt);
	
	self.i18n = i18n;
	
	self:registerControls(haGuiGeneralSettingsFrame.CONTROLS);

    return self;	
end;

function haGuiGeneralSettingsFrame:copyAttributes(src)
    haGuiGeneralSettingsFrame:superClass().copyAttributes(self, src)

    self.ui = src.ui
    self.i18n = src.i18n	
end

function haGuiGeneralSettingsFrame:initialize()
	self.backButtonInfo = {inputAction = InputAction.MENU_BACK};

    local elementBefore = self.guiShowHelp
    local elementAfter = self.guiNumHelper
    FocusManager:linkElements(self.guiClickSave, FocusManager.BOTTOM, elementBefore)
    FocusManager:linkElements(self.guiClickSave, FocusManager.TOP, elementAfter)
	
end;

function haGuiGeneralSettingsFrame:onFrameOpen()
    haGuiGeneralSettingsFrame:superClass().onFrameOpen(self);
	
	if helperadmin.debug then
		print("--- HelperAdminGui Debug ... haGuiGeneralSettingsFrame:onFrameOpen(haGui.actGuiNumHelper = "..tostring(haGui.actGuiNumHelper)..")");
	end;		
	
	-- Header
	self.frameHeader.elements[2]:setText(g_i18n:getText("gui_hat_GenSetting"));

	-- Num Helpers
	self.guiNumHelper.elements[4]:setText(g_i18n:getText("gui_hat_numHelper"));
	self.guiNumHelper.elements[6]:setText(g_i18n:getText("gui_hat_numHelperToolTip"));
	
    local elementText = {};
	local startval = helperadmin.minHelper
	local maxcount = ((helperadmin.maxHelper - startval) / 2) + 1;
	local index = 0;
	
    for i = 1, maxcount, 1 do
        elementText[i] = tostring(startval);
		startval = startval + 2;	
    end	
	self.guiNumHelper:setTexts(elementText);
	
	for i=1, #elementText, 1 do
		if (elementText[i] == tostring(haGui.actGuiNumHelper * 2)) then
			index = i;
			break
		end;
	end;
	
	if (index ~= 0) then
		self.guiNumHelper:setState(index, false);
	end;

	-- Price per Hour	
	self.guiPricePerHour.elements[4]:setText(g_i18n:getText("gui_hat_PricePerHour"));
	self.guiPricePerHour.elements[6]:setText(g_i18n:getText("gui_hat_PricePerHourToolTip"));
		
	local currencySymbol = Utils.getNoNil(g_i18n:getCurrencySymbol(true), '?');
    elementText = {};
    for i = 1, #haGuiGeneralSettingsFrame.pricePerHour, 1 do
        elementText[i] = ""..tostring(haGuiGeneralSettingsFrame.pricePerHour[i]).. " "..tostring(currencySymbol);
    end;
    self.guiPricePerHour:setTexts(elementText);	
	
	index = 0;
	for i=1, #haGuiGeneralSettingsFrame.pricePerHour, 1 do
		if (haGuiGeneralSettingsFrame.pricePerHour[i] == haGui.actPricePerHour) then
			index = i;
			break
		end;
	end;
	if (index ~= 0) then
		self.guiPricePerHour:setState(index, false);
	end;
	
	--Overtime Factor 1
	self.guiOvertimeFact1.elements[4]:setText(g_i18n:getText("gui_hat_OvertimeFact1"));
	self.guiOvertimeFact1.elements[6]:setText(g_i18n:getText("gui_hat_OvertimeFact1ToolTip"));
	
    elementText = {};
    for i = 1, #haGuiGeneralSettingsFrame.overTimeFact1, 1 do
		if (i == 1) then
			elementText[i] = g_i18n:getText("gui_hat_NoOvertimeFact");
		else
			elementText[i] = ""..tostring(haGuiGeneralSettingsFrame.overTimeFact1[i]).. "";
		end;
    end;
    self.guiOvertimeFact1:setTexts(elementText);		
	
	index = 0;
	for i=1, #haGuiGeneralSettingsFrame.overTimeFact1, 1 do
		if (haGuiGeneralSettingsFrame.overTimeFact1[i] >= (haGui.actOvertimeFact1-0.05)) and (haGuiGeneralSettingsFrame.overTimeFact1[i] <= (haGui.actOvertimeFact1+0.05)) then
			index = i;
			break
		end;
	end;
	if (index ~= 0) then
		self.guiOvertimeFact1:setState(index, false);
	end;
	
	--Overtime Factor 2
	self.guiOvertimeFact2.elements[4]:setText(g_i18n:getText("gui_hat_OvertimeFact2"));
	self.guiOvertimeFact2.elements[6]:setText(g_i18n:getText("gui_hat_OvertimeFact2ToolTip"));

    elementText = {};
    for i = 1, #haGuiGeneralSettingsFrame.overTimeFact2, 1 do
		if (i == 1) then
			elementText[i] = g_i18n:getText("gui_hat_NoOvertimeFact");
		else
			elementText[i] = ""..tostring(haGuiGeneralSettingsFrame.overTimeFact2[i]).. "";
		end;
    end;	
	self.guiOvertimeFact2:setTexts(elementText);
	
	index = 0;
	for i=1, #haGuiGeneralSettingsFrame.overTimeFact2, 1 do	
		if (haGuiGeneralSettingsFrame.overTimeFact2[i] >= (haGui.actOvertimeFact2-0.05)) and (haGuiGeneralSettingsFrame.overTimeFact2[i] <= (haGui.actOvertimeFact2+0.05)) then
			index = i;
			break
		end;
	end;
	if (index ~= 0) then
		self.guiOvertimeFact2:setState(index, false);
	end;

	-- HorseRiding
	self.guiRideHorses.elements[4]:setText(g_i18n:getText("gui_hat_RideHorses"));
	self.guiRideHorses.elements[6]:setText(g_i18n:getText("gui_hat_RideHorsesToolTip"));
	
	self.guiRideHorses:setIsChecked(haGui.actGuiRideHorses);	
	
	-- Show Help
	self.guiShowHelp.elements[4]:setText(g_i18n:getText("gui_hat_ShowHelp"));
	self.guiShowHelp.elements[6]:setText(g_i18n:getText("gui_hat_ShowHelpToolTip"));
	
	self.guiShowHelp:setIsChecked(haGui.actGuiShowHelp);
	
	-- Save Settings
	self.guiClickSave.elements[1]:setText(g_i18n:getText("gui_hat_Save"));
    self.guiClickSave.elements[1]:setImageFilename(nil, helperadmin.dir .. 'icons/button_save.png');
    self.guiClickSave.elements[1].icon.uvs = Overlay.DEFAULT_UVS;
	self.guiClickSave.elements[3]:setText(g_i18n:getText("gui_hat_SaveToolTip"));	

	if FocusManager:getFocusedElement() == nil then
		self:setSoundSuppressed(true)
		FocusManager:setFocus(self.boxLayout)
		self:setSoundSuppressed(false)
	end	
end;

function haGuiGeneralSettingsFrame:onClickNumHelper()
	haGui.actGuiNumHelper = self.guiNumHelper:getState() + 2;
	if helperadmin.debug then
		print("--- HelperAdminGui Debug ... haGuiGeneralSettingsFrame:onClickNumHelper(haGui.actGuiNumHelper = ".. tostring(haGui.actGuiNumHelper)..")");
	end;
end;

function haGuiGeneralSettingsFrame:onClickPricePerHour()
	local state = self.guiPricePerHour:getState();
	haGui.actPricePerHour = haGuiGeneralSettingsFrame.pricePerHour[state];
	if helperadmin.debug then
		print("--- HelperAdminGui Debug ... haGuiGeneralSettingsFrame:onClickPricePerHour(haGui.actPricePerHour = ".. tostring(haGui.actPricePerHour)..")");
	end;
end;

function haGuiGeneralSettingsFrame:onClickOvertimeFact1()
	local state = self.guiOvertimeFact1:getState();
	haGui.actOvertimeFact1 = haGuiGeneralSettingsFrame.overTimeFact1[state];
	if helperadmin.debug then
		print("--- HelperAdminGui Debug ... haGuiGeneralSettingsFrame:onClickOvertimeFact1(haGui.actOvertimeFact1 = ".. tostring(haGui.actOvertimeFact1)..")");
	end;
end;

function haGuiGeneralSettingsFrame:onClickOvertimeFact2()
	local state = self.guiOvertimeFact2:getState();
	haGui.actOvertimeFact2 = haGuiGeneralSettingsFrame.overTimeFact2[state];
	if helperadmin.debug then
		print("--- HelperAdminGui Debug ... haGuiGeneralSettingsFrame:onClickOvertimeFact2(haGui.actOvertimeFact2 = ".. tostring(haGui.actOvertimeFact2)..")");
	end;
end;

function haGuiGeneralSettingsFrame:onClickRideHorses()
	haGui.actGuiRideHorses = Utils.getNoNil(self.guiRideHorses:getIsChecked(), true);
	if helperadmin.debug then
		print("--- HelperAdminGui Debug ... haGuiGeneralSettingsFrame:onClickRideHorses(haGui.actGuiRideHorses = ".. tostring(haGui.actGuiRideHorses)..")");
	end;
end;

function haGuiGeneralSettingsFrame:onClickShowHelp()
	haGui.actGuiShowHelp = Utils.getNoNil(self.guiShowHelp:getIsChecked(), true);
	if helperadmin.debug then
		print("--- HelperAdminGui Debug ... haGuiGeneralSettingsFrame:onClickShowHelp(haGui.actGuiShowHelp = ".. tostring(haGui.actGuiShowHelp)..")");
	end;
end;


function haGuiGeneralSettingsFrame:onClickSaveSettings()
	if (haGui.actGuiNumHelper ~= nil) then
		helperadmin.nameMan = {};
		helperadmin.nameWoman = {};		
		for i = 1, haGui.actGuiNumHelper, 1 do
			if (haGui.actGuiNameMHelp[i] ~= nil) and (haGui.actGuiNameMHelp[i] ~= "") then
				helperadmin.nameMan[i] = haGui.actGuiNameMHelp[i];
			else
				helperadmin.nameMan[i] = "Man"..tostring(i);
			end;
			if (haGui.actGuiNameWHelp[i] ~= nil) and (haGui.actGuiNameWHelp[i] ~= "") then
				helperadmin.nameWoman[i] = haGui.actGuiNameWHelp[i];
			else
				helperadmin.nameWoman[i] = "Woman" ..i;
			end;
		end;
	end;

	helperadmin.pricePerHour = haGui.actPricePerHour;
	helperadmin.overtimeFact1 = haGui.actOvertimeFact1;
	helperadmin.overtimeFact2 = haGui.actOvertimeFact2;
	
	helperadmin.rideHorses = haGui.actGuiRideHorses;
	
	if helperadmin.debug then
		print("--- HelperAdmin Debug ... haGuiGeneralSettingsFrame:onClickSaveSettings(haGui.actGuiNumHelper = "..tostring(haGui.actGuiNumHelper)..")");
		print("--- HelperAdmin Debug ... haGuiGeneralSettingsFrame:onClickSaveSettings(helperadmin.nameMan)");
		DebugUtil.printTableRecursively(helperadmin.nameMan,"*n* ", 0, 1);
		print("--- HelperAdmin Debug ... haGuiGeneralSettingsFrame:onClickSaveSettings(helperadmin.nameWoman)");
		DebugUtil.printTableRecursively(helperadmin.nameWoman,"*n* ", 0, 1);
		print("--- HelperAdmin Debug ... haGuiGeneralSettingsFrame:onClickSaveSettings(haGui.actPricePerHour = "..tostring(haGui.actPricePerHour)..")");
		print("--- HelperAdmin Debug ... haGuiGeneralSettingsFrame:onClickSaveSettings(haGui.actOvertimeFact1 = "..tostring(haGui.actOvertimeFact1)..")");
		print("--- HelperAdmin Debug ... haGuiGeneralSettingsFrame:onClickSaveSettings(haGui.actOvertimeFact2 = "..tostring(haGui.actOvertimeFact2)..")");
		print("--- HelperAdmin Debug ... haGuiGeneralSettingsFrame:onClickSaveSettings(haGui.actGuiRideHorses = "..tostring(haGui.actGuiRideHorses)..")");
		print("--- HelperAdmin Debug ... haGuiGeneralSettingsFrame:onClickSaveSettings(haGui.actGuiShowHelp = "..tostring(haGui.actGuiShowHelp)..")");
	end;	

	--Update Show Help
	helperadmin.showHelp = haGui.actGuiShowHelp;
	for i=1, #helperadmin.eventName, 1 do
		if (g_inputBinding.events[helperadmin.eventName[i]] ~= nil) then
			g_inputBinding.events[helperadmin.eventName[i]].displayIsVisible = helperadmin.showHelp;
		end;	
	end;
	
	-- send data from gui to server
	haMultiplayerUpdEvent.sendEvent();	
end