--
-- HelperAdmin
-- V1.0.0.0
--
-- @author apuehri
-- @date 21/12/2021
--
-- Copyright (C) apuehri|LS-Modcompany
-- V1.0.0.0 ..... LS22 first implementation

helperadmin = {};
helperadmin.Version = "1.0.0.0";
helperadmin.debug = false;
helperadmin.dir = g_currentModDirectory;
helperadmin.minHelper = 6;
helperadmin.maxHelper = 20;

source(Utils.getFilename("gui/haGui.lua", helperadmin.dir));
source(Utils.getFilename("gui/haGuiGeneralSettingsFrame.lua", helperadmin.dir));
source(Utils.getFilename("gui/haGuiHelperNameFrame.lua", helperadmin.dir));
source(Utils.getFilename("scripts/helperAdminTools.lua", helperadmin.dir));
source(Utils.getFilename("scripts/utils.lua", helperadmin.dir));

addModEventListener(helperadmin);

function helperadmin.prerequisitesPresent(specializations)
    return true;
end;

function helperadmin:loadMap(name)
	if helperadmin.debug then
		print("--- HelperAdmin Debug ... helperadmin:loadMap ++ isClient="..tostring(g_currentMission:getIsClient()).." ,isServer="..tostring(g_currentMission:getIsServer()).." ,isMasterUser="..tostring(g_currentMission.isMasterUser).." ,isMultiplayer="..tostring(g_currentMission.missionDynamicInfo.isMultiplayer).." ---");
	end;
    -- Only needed for action event for player
    Player.registerActionEvents = Utils.appendedFunction(Player.registerActionEvents, helperadmin.registerActionEventsPlayer);
	Player.removeActionEvents = Utils.appendedFunction(Player.removeActionEvents, helperadmin.removeActionEventsPlayer);		
		
	-- SaveSettings
	FSBaseMission.saveSavegame = Utils.appendedFunction(FSBaseMission.saveSavegame, helperadmin.saveSettings);

	--initialize
	helperadmin.nameMan = {};
	helperadmin.nameWoman = {};
	helperadmin.newNumHirables = 6;
	helperadmin.showHelp = true;
	helperadmin.eventName = {};
	helperadmin.pricePerHour = 1000.0;
	helperadmin.overtimeFact1 = 1.8;
	helperadmin.overtimeFact2 = 3.4;
	helperadmin.rideHorses = true;
	
	--load Savegame
	local savegameIndex = g_currentMission.missionInfo.savegameIndex;
	local savegameFolderPath = g_currentMission.missionInfo.savegameDirectory;
	if savegameFolderPath == nil then
		savegameFolderPath = ('%ssavegame%d'):format(getUserProfileAppPath(), savegameIndex);
	end;

	if g_currentMission:getIsServer() then
		if fileExists(savegameFolderPath .. '/careerSavegame.xml') then
			if fileExists(savegameFolderPath .. '/helperAdmin.xml') then
				print("--- loading HelperAdmin V".. helperadmin.Version .. " (c) by apuehri--- loading savegame ---");
				local key = "helperAdmin";
				local xmlFile = loadXMLFile("helperAdmin", savegameFolderPath .. "/helperAdmin.xml", key);			
				if xmlFile ~= nil then
					for i=1, helperadmin.maxHelper do
						-- man
						local XmlNameMan = getXMLString(xmlFile, key .."#nameMan"..i);
						if XmlNameMan ~= nil then
							helperadmin.nameMan[i] = XmlNameMan;
						end;
						-- woman
						local XmlNameWoman = getXMLString(xmlFile, key .."#nameWoman"..i);
						if XmlNameWoman ~= nil then
							helperadmin.nameWoman[i] = XmlNameWoman;
						else
							break;						
						end;
					end;
					-- ShowHelp
					helperadmin.showHelp = Utils.getNoNil(getXMLBool(xmlFile, key.."#showHelp"), true);
					-- PricePerHour
					helperadmin.pricePerHour = Utils.getNoNil(getXMLFloat(xmlFile, key.."#pricePerHour"), 1000.0);
					-- Overtime Factors
					helperadmin.overtimeFact1 = Utils.getNoNil(getXMLFloat(xmlFile, key.."#overtimeFact1"), 1.8);
					helperadmin.overtimeFact1 = math.round(helperadmin.overtimeFact1, 0.1);
					helperadmin.overtimeFact2 = Utils.getNoNil(getXMLFloat(xmlFile, key.."#overtimeFact2"), 3.4);
					helperadmin.overtimeFact2 = math.round(helperadmin.overtimeFact2, 0.1);
					-- HorseRiding
					helperadmin.rideHorses = Utils.getNoNil(getXMLBool(xmlFile, key.."#rideHorses"), true);					
				end;
				delete(xmlFile);
			else
				helperadmin.nameMan = {"Andy";"Phillip";"Franz";};
				helperadmin.nameWoman = {"Betty";"Sarah";"Elisabeth";};
				print("--- loading HelperAdmin V".. helperadmin.Version .. " (c) by apuehri--- loading initialvalues ---");			
			end;		
		else
			helperadmin.nameMan = {"Andy";"Phillip";"Franz";};
			helperadmin.nameWoman = {"Betty";"Sarah";"Elisabeth";};
			print("--- loading HelperAdmin V".. helperadmin.Version .. " (c) by apuehri--- loading initialvalues ---");	
		end;	

		-- register new helpers
		if g_currentMission:getIsServer() and (g_currentMission ~= nil) and (HelperManager ~= nil) and (g_helperManager ~= nil) then
			helperadmin:regNewHelpers();
		end;
	end;
		
	--gui
	local generalSettingsFrame = haGuiGeneralSettingsFrame:new(g_i18n);
	local helperNameFrame = haGuiHelperNameFrame:new(g_i18n);

    helperadmin.gui = haGui:new(g_messageCenter, g_i18n, g_inputBinding);
	
	g_gui:loadGui(helperadmin.dir .. "gui/haGuiGeneralSettingsFrame.xml", "haGuiGeneralSettingsFrame", generalSettingsFrame, true);
	g_gui:loadGui(helperadmin.dir .. "gui/haGuiHelperNameFrame.xml", "haGuiHelperNameFrame", helperNameFrame, true);
    g_gui:loadGui(helperadmin.dir .. "gui/haGui.xml", "haGui", helperadmin.gui);

	--change AIJob Prices
	helperAdminTools:changeAIJob();
	
	--HorseRiding
	g_messageCenter:subscribe(MessageType.HOUR_CHANGED, helperAdminTools.rideHorses, helperadmin);
end;

function helperadmin:loadSavegame() 
end;

function helperadmin:registerActionEventsPlayer()
	-- HelperAdmin Setting	
	local result, eventName = InputBinding.registerActionEvent(g_inputBinding, 'hat_setting',self, helperadmin.actionHelperadmin_Setting ,false ,true ,false ,true)
	if result then
		table.insert(helperadmin.eventName, eventName);
        g_inputBinding.events[eventName].displayIsVisible = helperadmin.showHelp;
    end
	if helperadmin.debug then
		print("--- HelperAdmin Debug ... helperadmin:registerActionEventsPlayer(helperadmin.eventName)");
		DebugUtil.printTableRecursively(helperadmin.eventName,"----",0,1)
	end;	
end;

function helperadmin:removeActionEventsPlayer()
	helperadmin.eventName = {};
	if helperadmin.debug then
		print("--- HelperAdmin Debug ... helperadmin:removeActionEventsPlayer(helperadmin.eventName)");
		DebugUtil.printTableRecursively(helperadmin.eventName,"----",0,2)
	end;
end;

function helperadmin:mouseEvent(posX, posY, isDown, isUp, button)
end;

function helperadmin:keyEvent(unicode, sym, modifier, isDown)
end;

function helperadmin:update(dt)
end;

function helperadmin:draw()   
end;

function helperadmin:regNewHelpers()
	if (#helperadmin.nameMan == #helperadmin.nameWoman) then
		if helperadmin.debug then
			print("--- HelperAdmin Debug --- helperadmin:RegNewHelpers List of old Helpers........");
			DebugUtil.printTableRecursively(g_helperManager,"--- HelperAdmin Debug --- ", 0, 5);		
		end;		
		
		helperadmin.newNumHirables = #helperadmin.nameMan + #helperadmin.nameWoman;
		if (helperadmin.newNumHirables ~= nil) and (helperadmin.newNumHirables >= helperadmin.minHelper) and (helperadmin.newNumHirables <= helperadmin.maxHelper) then
			g_currentMission.maxNumHirables	= helperadmin.newNumHirables;
			if helperadmin.debug then
				print("--- HelperAdmin Debug ... helperadmin:RegNewHelpers(g_currentMission.maxNumHirables) = ".. g_currentMission.maxNumHirables);
			end;
		else
			print("--- HelperAdmin Error ... Number of helpers too low (min " ..tostring(helperadmin.minHelper)..") or too high (max" ..tostring(helperadmin.maxHelper).."), no changes have been made !! ---");
			return;
		end;
		
		g_helperManager:initDataStructures(); --Init Datas		
		
		-- Register new Helpers
		-- Info helperxx.xml
		-- 		man (aiHelper01.xml,aiHelper02.xml,aiHelper03.xml,aiHelper04.xml,aiHelper07.xml,aiHelper10.xml)
		-- 		woman (aiHelper05.xml,aiHelper06.xml,aiHelper07.xml,aiHelper09.xml)
		local HelperManXml = {[1]="dataS/character/humans/npc/aiHelpers/aiHelper01.xml",[2]="dataS/character/humans/npc/aiHelpers/aiHelper02.xml",[3]="dataS/character/humans/npc/aiHelpers/aiHelper03.xml",[4]="dataS/character/humans/npc/aiHelpers/aiHelper04.xml",[5]="dataS/character/humans/npc/aiHelpers/aiHelper07.xml"};
		local HelperManColor = {[1]="1 0 0",[2]="0 1 0",[3]="0 0 1",[4]="1 1 0",[5]="0.5 1 0.5"};
		local HelperWomanXml = {[1]="dataS/character/humans/npc/aiHelpers/aiHelper05.xml",[2]="dataS/character/humans/npc/aiHelpers/aiHelper06.xml",[3]="dataS/character/humans/npc/aiHelpers/aiHelper08.xml",[4]="dataS/character/humans/npc/aiHelpers/aiHelper09.xml",[5]="dataS/character/humans/npc/aiHelpers/aiHelper05.xml"};	
		local HelperWomanColor = {[1]="1 0 1",[2]="0 1 1",[3]="0.5 0.5 1",[4]="1 0.5 0.5",[5]="0.5 0.5 0.5"};

		local manNameIndex = 1;
		local womanNameIndex = 1;
				
		for i=1, g_currentMission.maxNumHirables do
			local calcmod = math.fmod(i,2);
			local maxNameIndex = 0;
			if calcmod == 1 then				
				local xmlIndex = math.random(1,5);
				local color = string.getVectorN(HelperManColor[i], 3) or {1, 1, 1};
				g_helperManager:addHelper(helperadmin.nameMan[manNameIndex], string.sub(helperadmin.nameMan[manNameIndex], 1, 1), color, HelperManXml[xmlIndex], nil, true);
				maxNameIndex = #helperadmin.nameMan;
				if manNameIndex < maxNameIndex then
					manNameIndex = manNameIndex + 1;
				else
					manNameIndex = 1;
				end;				
			else				
				local xmlIndex = math.random(1,5);
				local color = string.getVectorN(HelperWomanColor[i], 3) or {1, 1, 1};
				g_helperManager:addHelper(helperadmin.nameWoman[womanNameIndex], string.sub(helperadmin.nameWoman[womanNameIndex], 1, 1), color, HelperWomanXml[xmlIndex], nil, true);
				maxNameIndex = #helperadmin.nameWoman;
				if womanNameIndex < maxNameIndex then
					womanNameIndex = womanNameIndex + 1;
				else
					womanNameIndex = 1;
				end;
			end;	
		end;

		if helperadmin.debug then
			print("--- HelperAdmin Debug --- helperadmin:RegNewHelpers List of new Helpers........");
			DebugUtil.printTableRecursively(g_helperManager,"--- HelperAdmin Debug --- ", 0, 5);		
		end;
	else
		print("--- HelperAdmin Error ... Number of male and female helpers is different, no changes have been made !! ---");
	end;
end;

function helperadmin:saveSettings()
	local savegameIndex = g_currentMission.missionInfo.savegameIndex;
	local savegameFolderPath = g_currentMission.missionInfo.savegameDirectory;
	if savegameFolderPath == nil then
		savegameFolderPath = ('%ssavegame%d'):format(getUserProfileAppPath(), savegameIndex);
	end;
	
	local key = "helperAdmin";
	local xmlFile = createXMLFile("helperAdmin", savegameFolderPath .. "/helperAdmin.xml", key);
	
	for i=1, #helperadmin.nameMan do			
		setXMLString(xmlFile, key .."#nameMan"..i, helperadmin.nameMan[i]);
	end;
	for i=1, #helperadmin.nameWoman do
		setXMLString(xmlFile, key .."#nameWoman"..i, helperadmin.nameWoman[i]);
	end;
	-- ShowHelp
	setXMLBool(xmlFile, key .. "#showHelp", helperadmin.showHelp);
	-- PricePerHour
	setXMLFloat(xmlFile, key.."#pricePerHour", helperadmin.pricePerHour);
	-- Overtime Factors
	setXMLFloat(xmlFile, key.."#overtimeFact1", helperadmin.overtimeFact1);
	setXMLFloat(xmlFile, key.."#overtimeFact2", helperadmin.overtimeFact2);
	-- HorseRiding
	setXMLBool(xmlFile, key .. "#rideHorses", helperadmin.rideHorses);	
	

	saveXMLFile(xmlFile);
	delete(xmlFile);	
end;

function helperadmin:actionHelperadmin_Setting(actionName, keyStatus, arg3, arg4, arg5)
	if g_currentMission.isMasterUser then
		if (#g_helperManager.availableHelpers == g_helperManager.numHelpers) then
			if g_gui.currentGui == nil then		
				g_gui:showGui("haGui")
			end
		end;
	end;
end;

-- *****+++++*****+++++ Multiplayer *****+++++*****+++++
local origServerSendObjects = Server.sendObjects;

function Server:sendObjects(connection, x, y, z, viewDistanceCoeff)
	connection:sendEvent(haMultiplayerJoinEvent:new());
	return origServerSendObjects(self, connection, x, y, z, viewDistanceCoeff);
end;

haMultiplayerJoinEvent = {};
haMultiplayerJoinEvent_mt = Class(haMultiplayerJoinEvent, Event);
InitEventClass(haMultiplayerJoinEvent, 'haMultiplayerJoinEvent');

function haMultiplayerJoinEvent:emptyNew()
	local self = Event.new(haMultiplayerJoinEvent_mt);
	self.className = 'helperadmin.haMultiplayerJoinEvent';
	return self;
end;

function haMultiplayerJoinEvent:new()
	local self = haMultiplayerJoinEvent.emptyNew()
	return self;
end;

-- Send data from the server to the client haMultiplayerJoinEvent
function haMultiplayerJoinEvent:writeStream(streamId, connection)
	if not connection:getIsServer() then	
		if helperadmin.debug then
			print("--- HelperAdmin Debug ... sending data to joining client: ..... ---");
			print("--- HelperAdmin Debug ... haMultiplayerJoinEvent:writeStream(g_currentMission.maxNumHirables = "..tostring(helperadmin.newNumHirables)..")");
		end;
		streamWriteUInt8(streamId, helperadmin.newNumHirables);
		for i = 1, (helperadmin.newNumHirables / 2) do
			streamWriteString(streamId, helperadmin.nameMan[i]);
			streamWriteString(streamId, helperadmin.nameWoman[i]);
		end;
		streamWriteFloat32(streamId, helperadmin.pricePerHour);
		streamWriteFloat32(streamId, helperadmin.overtimeFact1);
		streamWriteFloat32(streamId, helperadmin.overtimeFact2);
		streamWriteBool(streamId, helperadmin.showHelp);
		streamWriteBool(streamId, helperadmin.rideHorses);		
	end;	
end;

-- Read from the server haMultiplayerJoinEvent
function haMultiplayerJoinEvent:readStream(streamId, connection)
	if connection:getIsServer() then
		g_currentMission.maxNumHirables = streamReadUInt8(streamId);
		for i = 1, (g_currentMission.maxNumHirables / 2) do
			helperadmin.nameMan[i] = streamReadString(streamId);
			helperadmin.nameWoman[i] = streamReadString(streamId);
		end;
		helperadmin.pricePerHour = streamReadFloat32(streamId);
		helperadmin.overtimeFact1 = math.round(streamReadFloat32(streamId), 0.1);
		helperadmin.overtimeFact2 = math.round(streamReadFloat32(streamId), 0.1);
		helperadmin.showHelp = streamReadBool(streamId);
		helperadmin.rideHorses = streamReadBool(streamId);
		if helperadmin.debug then
			print("--- HelperAdmin Debug ... reading data from server: ..... ---");
			print("--- HelperAdmin Debug ... haMultiplayerJoinEvent:readStream(g_currentMission.maxNumHirables = "..tostring(g_currentMission.maxNumHirables)..")");
		end;
		-- init Gui
		haGui:initData();
		
		-- register new helpers
		if (g_currentMission ~= nil) and (HelperManager ~= nil) and (g_helperManager ~= nil) then
			helperadmin:regNewHelpers();
		end;		
	end;
end;


haMultiplayerUpdEvent = {};
haMultiplayerUpdEvent_mt = Class(haMultiplayerUpdEvent, Event);
InitEventClass(haMultiplayerUpdEvent, 'haMultiplayerUpdEvent');

function haMultiplayerUpdEvent:emptyNew()
	local self = Event.new(haMultiplayerUpdEvent_mt);
	self.className = 'helperadmin.haMultiplayerUpdEvent';
	return self;
end;

function haMultiplayerUpdEvent:new()
	local self = haMultiplayerUpdEvent.emptyNew()
	return self;
end;

-- Send data from the client to the server haMultiplayerUpdEvent
function haMultiplayerUpdEvent:writeStream(streamId, connection)
	if connection:getIsServer() then	
		if helperadmin.debug then
			print("--- HelperAdmin Debug ... sending data to server: ..... ---");
		end;
		streamWriteUInt8(streamId, haGui.actGuiNumHelper);
		for i = 1, haGui.actGuiNumHelper do
			streamWriteString(streamId, helperadmin.nameMan[i]);
			streamWriteString(streamId, helperadmin.nameWoman[i]);
		end;
		streamWriteFloat32(streamId, helperadmin.pricePerHour);
		streamWriteFloat32(streamId, helperadmin.overtimeFact1);
		streamWriteFloat32(streamId, helperadmin.overtimeFact2);
		streamWriteBool(streamId, helperadmin.showHelp);
		streamWriteBool(streamId, helperadmin.rideHorses);		
	end;	
end;

-- Read from the client haMultiplayerUpdEvent
function haMultiplayerUpdEvent:readStream(streamId, connection)
	if not connection:getIsServer() then
		if helperadmin.debug then
			print("--- HelperAdmin Debug ... reading data from client: ..... ---");
		end;
		local newNumHelpGui = streamReadUInt8(streamId);
		for i = 1, newNumHelpGui do
			helperadmin.nameMan[i] = streamReadString(streamId);
			helperadmin.nameWoman[i] = streamReadString(streamId);
		end;
		helperadmin.pricePerHour = streamReadFloat32(streamId);
		helperadmin.overtimeFact1 = math.round(streamReadFloat32(streamId), 0.1);
		helperadmin.overtimeFact2 = math.round(streamReadFloat32(streamId), 0.1);
		helperadmin.showHelp = streamReadBool(streamId);
		helperadmin.rideHorses = streamReadBool(streamId);		
	end;
end;

function haMultiplayerUpdEvent:sendEvent()
	if g_currentMission.missionDynamicInfo.isMultiplayer and not g_currentMission:getIsServer() then
        -- Send to server
        g_client:getServerConnection():sendEvent(haMultiplayerUpdEvent:new());
	end;
end
